/* $Id$ */

import org.codehaus.groovy.grails.commons.ApplicationHolder
import org.codehaus.groovy.grails.commons.GrailsClassUtils
import org.codehaus.groovy.grails.commons.ConfigurationHolder as CH
import com.atlassian.crowd.integration.http.HttpAuthenticator
import com.atlassian.crowd.integration.service.soap.client.SecurityServerClient


class CrowdAuthSecurityFilters {
    HttpAuthenticator httpAuthenticator
    SecurityServerClient securityServerClient

    def filters = {
        log.info("Initializing Crowd Auth Security Filters")

        authenticationCheck(controller: '*', action: '*') {
            before = {
                // controllerName and actionName are given to us by Spring/Grails

                def application= ApplicationHolder.application
                def applicationControllerClasses= application.controllerClasses

                def controllerClass = applicationControllerClasses.find() { it.logicalPropertyName == controllerName }
                if (!controllerClass) {
                    log.warn("Couldn't find a Controller class associated with '${controllerName}'")
                    return true;
                }
                else log.debug("Controller class '${controllerClass}' located")

                def requireAuthentication= false

                // Process default authenticate
                def defaultAuthenticate= CH.config.crowdAuth.defaultAuthenticate
                log.debug("DefaultAuthenticate: ${defaultAuthenticate}")
                switch (defaultAuthenticate) {
                case Boolean:
                    requireAuthentication= defaultAuthenticate
                    break
                case Map:
                    def except= defaultAuthenticate['except']
                    def only= defaultAuthenticate['only']

                    if (except) {
                        if ((actionName? actionName : 'index') in except)
                            requireAuthentication= false
                        else
                            requireAuthentication= true
                    }
                    else if (only) {
                        if ((actionName? actionName : 'index') in only)
                            requireAuthentication= true
                        else
                            requireAuthentication= false
                    }
                    else {
                        log.warn("DefaultAuthenticate was not correctly defined in configuration.")
                    }
                    break
                default:
                    log.warn("DefaultAuthenticate was not correctly defined in configuration.")
                }

                // Process Controller/Action supplied authenticate
                Object authenticate= GrailsClassUtils.getStaticPropertyValue(controllerClass.getClazz(), "authenticate");
                switch (authenticate) {
                case Boolean:
                    requireAuthentication= authenticate
                    break
                case Map:
                    def except= authenticate['except']
                    def only= authenticate['only']

                    if (except) {
                        if ((actionName? actionName : 'index') in except)
                            requireAuthentication= false
                        else
                            requireAuthentication= true
                    }
                    else if (only) {
                        if ((actionName? actionName : 'index') in only)
                            requireAuthentication= true
                        else
                            requireAuthentication= false
                    }
                    else {
                        log.warn("Authenticate was not correctly defined in Controller '${controllerClass}'")
                    }
                    break
                default:
                    log.warn("Authenticate was not correctly defined in Controller '${controllerClass}'")
                }

                log.debug("requireAuthentication: ${requireAuthentication}")
                if (requireAuthentication) {
                    // Is user's session authentication valid?
                    if (!CrowdAuthUtils.isAuthenticated(request)) {
                        log.debug("User is not authenticated at application level")

                        // Does the user have Crowd SSO authentication?
                        if (httpAuthenticator.isAuthenticated(request, response)) {
                            // The user has Crowd SSO authentication
                            // TODO: Catch Exceptions?
                            session.principal= httpAuthenticator.getPrincipal(request)
                            session.principalGroups= securityServerClient.findGroupMemberships(session.principal.name).toList()
                            session.principalRoles= securityServerClient.findRoleMemberships(session.principal.name).toList()
                        }
                        else {
                            // The user does not have Crowd SSO authentication
                            log.debug("User is not authenticated at Crowd SSO level")

                            def originalRequestParams = [controller: controllerName, action: actionName? actionName : 'index']
                            originalRequestParams.putAll(params)
                            session.originalRequestParams = originalRequestParams

                            // User is not authenticated, redirect the user to login
                            if (CH.config.crowdAuth.loginAction) {
                                log.debug("Redirecting to loginController, loginAction")
                                redirect(controller: CH.config.crowdAuth.loginController, action: CH.config.crowdauth.loginAction)
                                return
                            }
                            else if (CH.config.crowdAuth.loginController) {
                                log.debug("Redirecting to loginController")
                                redirect(controller: CH.config.crowdAuth.loginController)
                                return
                            }
                            else if (CH.config.crowdAuth.loginUrl) {
                                log.debug("Redirecting to loginUrl")
                                redirect(uri: CH.config.crowdAuth.loginUrl)
                                return
                            }
                            else {
                                log.error("No login method is configured. Please set one of loginController [and loginAction] or loginUrl.")
                                return false
                            }
                        }
                    }
                }
            }
        }

        authorizationCheck(controller: '*', action: '*') {
            before = {
                def application= ApplicationHolder.application
                def applicationControllerClasses= application.controllerClasses

                def controllerClass = applicationControllerClasses.find() { it.logicalPropertyName == controllerName }
                if (!controllerClass) {
                    log.warn("Couldn't find a Controller class associated with '${controllerName}'")
                    return true;
                }
                else log.debug("Controller class '${controllerClass}' located")

                List requireAuthorization= []

                // Process Controller/Action supplied authenticate
                def defaultAuthorize= CH.config.crowdAuth.defaultAuthorize
                switch (defaultAuthorize) {
                case String:
                    requireAuthorization= [defaultAuthorize]
                    break
                case List:
                    requireAuthorization= defaultAuthorize
                    break
                case Map:
                    def all= defaultAuthorize['all']
                    if (all) {
                        switch (all) {
                        case String:
                            requireAuthorization.add(all)
                            break
                        case List:
                            requireAuthorization.addAll(all)
                        default:
                            log.warn("DefaultAuthorize was not correctly defined in configuration.")
                        }
                    }

                    def thisAction= defaultAuthorize[actionName? actionName : 'index']
                    if (thisAction) {
                        switch (thisAction) {
                        case String:
                            requireAuthorization.add(thisAction)
                            break
                        case List:
                            requireAuthorization.addAll(thisAction)
                            break
                        default:
                            log.warn("DefaultAuthorize was not correctly defined in configuration.")
                        }
                    }
                }

                // Process Controller/Action supplied authorize
                Object authorize= GrailsClassUtils.getStaticPropertyValue(controllerClass.getClazz(), "authorize");
                switch (authorize) {
                case String:
                    requireAuthorization= [authorize]
                    break
                case List:
                    requireAuthorization= authorize
                    break
                case Map:
                    def all= authorize['all']
                    if (all) {
                        switch (all) {
                        case String:
                            requireAuthorization.add(all)
                            break
                        case List:
                            requireAuthorization.addAll(all)
                        default:
                            log.warn("Authorize was not correctly defined in Controller '${controllerClass}'.")
                        }
                    }

                    def thisAction= authorize[actionName? actionName : 'index']
                    if (thisAction) {
                        switch (thisAction) {
                        case String:
                            requireAuthorization.add(thisAction)
                            break
                        case List:
                            requireAuthorization.addAll(thisAction)
                            break
                        default:
                            log.warn("Authorize was not correctly defined in Controller '${controllerClass}'.")
                        }
                    }
                }

                log.debug("requireAuthorization: ${requireAuthorization}")
                if (requireAuthorization) {
                    if (!session.principalGroups?.containsAll(requireAuthorization)) {
                        response.sendError(403)
                    }
                }
            }
        }
    }
}

